/*************************************************************************************
__MXSDOC__
Author:				Gavyn Thompson
Company:				GTVFX
Website:				www.gtvfx.com
Email:				gthompson@gtvfx.com
ScriptVersion:			v1.00
Updated:				12/31/2013
[Purpose]
Install for individual tool macro scripts.
__END__
*************************************************************************************/
struct macroInstall_lib
(
	self,
	ro,
	tempDir = ((getDir #temp)+@"\GTVFX"),
	scriptsDir = ((getDir #scripts)+@"\GTVFX\"),
	macrosDir = ((getDir #userMacros)+"\\"),
	tempGTVFX = (tempDir + @"\Scripts\GTVFX"),
	sFileName = getFileNameFile (getFiles (tempGTVFX + "\\*.ms"))[1],
		
	fn removeTempDir =
	(
		if doesFileExist tempDir then dosCommand ("rmdir \"" + tempDir + "\" /s /q")
	),
	fn copyFiles_FN fromDir toDir =
	(
		if not doesFileExist fromDir then
		(
			return false
		)
		else
		(		
			sFileArr = getFiles (fromDir + @"\*.ms")
			for i in sFileArr do
			(
				destFile = (toDir+(getFileNameFile i)+(getFileNameType i))
				if doesFileExist destFile then deleteFile destFile
				copyFile i destFile
			)
		)
		true
	),
	fn install_FN =
	(
		if not (makeDir scriptsDir) then return false
		if not copyFiles_FN tempGTVFX scriptsDir then return false
		true
	),
	fn uninstall_FN arr:#() =
	(
		if doesFileExist macrosDir then arr += getFiles (macrosDir + sFileName + "*")
		if doesFileExist scriptsDir then arr += getFiles (scriptsDir + sFileName + "*")
		if arr.count != 0 then for i in arr do deleteFile i
	),
	fn formatMacro_FN macroName mcrCategory sFile stream =
	(
		format ("macroScript %\ncategory:\"%\"\nbuttonText:\"%\"\n(\n\tfileIn ((getDir #scripts)+\"\\\\GTVFX\\\\\"+\"%\"+\".ms\")\n)\n") \
		macroName mcrCategory macroName sFile to:stream
	),
	fn createMacro =
	(
		sFile = (getFiles (scriptsDir + sFileName + "*.ms"))[1]
		fileName = substituteString (getFileNameFile sFile) "GTVFX-" ""	
		stream = stringStream ""
		formatMacro_FN fileName "GTVFX" sFileName stream
		seek stream 0
		readExpr stream
	),
	fn ui =
	(
		rollout ro "GTVFX: Install"
		(
			local self
			button btn_install "Install Macro" width:230 height:30
			button btn_unInstall "Uninstall Macro" width:230 height:30
			hyperLink hyp_website "www.gtvfx.com" color:orange  hoverColor:red visitedColor:orange address:"http://www.gtvfx.com" pos:[85,75]
			fn _init pself=
			(
				self = pself
			)
			on btn_install pressed do
			(
				if self.install_FN() then
				(
					self.createMacro()
					self.removeTempDir()
					messageBox "GTVFX macroscript successfully installed.\nYou may need to restart Max if you are updating to a newer version of the macro." title:"GTVFX: Success :)"
				)
				else messageBox "Could not install the tool.\nDouble check that you have permissions to copy files to the 3ds Max root directory." title:"GTVFX: Error :("
				try(destroyDialog self.ro)catch() 
			)
			on btn_unInstall pressed do
			(
				if queryBox "Are you sure you want to remove the macroscript? :(" then
				(
					self.uninstall_FN()
					self.removeTempDir()
					try(destroyDialog self.ro)catch()
					messageBox "Macroscript removed." title:"GTVFX: :("
				)
			)
		)
		createDialog ro width:250 height:95
	),
	fn _init =
	(
		self = this
		ui()
		ro._init self
	),
	init = _init()
)
macroInit = macroInstall_lib()